/* bubble_plot.sas */

/*
 Disclaimer: No guarantee as to suitability or accuracy is
 given or implied. User uses this code entirely at their own risk.
*/

/* Omit the date and page number to get clean PDF output */
options label nodate nonumber;

/* At this point assume that the data is "ready" */
%macro doWMWbub(figname,dset,outvar,grpvar,grp1val,grp2val,grp1name,grp2name,titl);

proc datasets;
  delete combo;
run;
/*  1 Copy Dataset, 2 Delete Missings, 3 Split or New Outcome, 4 Format */
data group1; set &dset;
  if &grpvar=&grp1val;
  rename 
    &outvar = &outvar._a
   ;
run;
data group2; set &dset;
  if &grpvar=&grp2val;
  rename 
    &outvar = &outvar._b
   ;
run;

proc sql;
   title 'All Possible Connections';
   create table combo as
   select *
       from group1 , group2
	   ;
run;
proc freq data=combo;
  table &outvar._a*&outvar._b / list missing;
  ods output list=xtab_counts;
run;
title;
proc means data=&dset min max noprint;
  var &outvar;
  output out=minmax min=min max=max;
run;

data x; set minmax;
  min=min-1;
  max=max+1;
  call symput('min',min);
  call symput('max',max);
run;
data line; set x;
  x = min; y1=min; output;
  x = max; y1=max; output;
run;

/* "Trick" SAS into scaling the bubble size better */
data fake;
  y_a=0;
  y_b=0;
  frequency=0.0001;
run;

data xtab2;
  set xtab_counts line fake;
run;

proc format;
  value hide 0.0001='';
run;

title;

ods pdf file="M:\gdivine1\sasdirs\tech\Wilcoxon_paper3\Version4 Stuff\final_figures\&figname..pdf";
ods pdf file="&figname..pdf";
proc sgplot data=xtab2 noautolegend;
  bubble x=&outvar._a y=&outvar._b size=frequency / bradiusmin=0 
   datalabel=frequency datalabelattrs=(weight=BOLD size=12) 
   nooutline fillattrs= (color=bigb);
   series x=x y=y1/ lineattrs=(color=black)legendlabel="";
  xaxis label="&grp1name" ;
  yaxis label="&grp2name" ;
*  Title1 justify=left "&titl";
  format frequency hide.;
run;
ods pdf close;
%mend doWMWbub;

title;

%doWMWbub(figname=Figure6,dset=toy,grpvar=grp,outvar=y,
grp1val=1,grp2val=2,
grp1name=Group X,grp2name=Group Y);
*titl=%str(Figure 6 Bubble Plot Representing Estimate of p'' = Pr(X<Y) + Pr(X=Y)/2 for Toy Example);
/*
proc print data=xtab2 noobs;
  var y_a y_b frequency;
  where frequency>0;
run;

proc sgplot data=xtab2 noautolegend;
  bubble x=y_a y=y_b size=frequency / bradiusmin=0 
   datalabel=frequency datalabelattrs=(weight=BOLD size=12) 
   nooutline fillattrs= (color=bigb);
   series x=x y=y1/ lineattrs=(color=black)legendlabel="";
  xaxis label="&grp1name" ;
  yaxis label="&grp2name" ;
  Title1 justify=left "&titl";
  format frequency hide.;
run;
*/
