/* dominance.sas */

/*
 Disclaimer: No guarantee as to suitability or accuracy is
 given or implied. User uses this code entirely at their own risk.
*/

%macro dominance(figname,dset,outvar,grpvar,grp1name,grp2name,title);
/* Stuff to do a "Dominance Diagram" */
/*  First get the ranks for the two groups combined */
proc rank data=&dset out=ranked_dataset;
  var &outvar;
  ranks outvar_rank;
run;
proc sort data=ranked_dataset;
  by &grpvar outvar_rank;
run;
/* Next, get the _within_ group ranks */
data grp1 ; set ranked_dataset;
  if _n_=1 then group1=&grpvar;
  retain group1;
  if &grpvar=group1 then do; grp1rank=outvar_rank; output grp1; end;
run;
data grp2; set ranked_dataset;
  if _n_=1 then group1=&grpvar;
  retain group1;
  if &grpvar^=group1 then do; grp2rank=outvar_rank; output grp2; end;
run;
data grp1; set grp1;
  grprank1=_n_;
run;
data grp2; set grp2;
  grprank2=_n_;
run;
/* 
At this point we have two sets of ranks:
  grp1rank and grp2rank are the overall ranks from 1 to N=n1+n2
  grprank1 and grprank2 are the ranks within each group
 NOTE: grprank1 and grprank2 "ignore" ties (that is, ties are arbitrarily broken
*/

/* Make Sure Combo Does Not Already Exist */
proc datasets noprint;
  delete combo;
run;

proc sql;
   title 'All Possible Comparisons';
   create table combo as
   select *
       from grp1, grp2
           ;
run;
/* Create the variable for the symbols to be plotted */
data combo; set combo;
  if grp1rank<grp2rank then symbol=-1;
  if grp1rank=grp2rank then symbol=0;
  if grp1rank>grp2rank then symbol=1;
run;
proc format;
  value order -1="&grp1name > &grp2name" 0="&grp1name = &grp2name" 1="&grp1name < &grp2name";
run;

proc sort data=combo;
  by symbol;
run;

/* Do the Dominance Diagram */
/*  Note a nice size for the boxes may need to be found by trial and error */
goptions reset;
symbol1 i=none font=marker h=1 v=U color=black;
symbol2 i=none font=marker h=1 v=U color=grey;
symbol3 i=none h=1 v=point color=grey;
axis1 label=(a=90 r=0 h=2 "&grp2name Ranks") ;*order=0.5 to 4.5 by 0.5;
axis2 label=(h=2 "&grp1name Ranks") ;*order=0.5 to 4.5 by 0.5;
title;
ods pdf file="&figname..pdf";
proc gplot data=combo;
  plot grprank1*grprank2=symbol / vaxis=axis1 haxis=axis2;
  format symbol order.;
  title1 "&title Dominance Diagram";
run;
title3;
ods pdf close;
%mend dominance;

%dominance(figname=EDVisit_domdiag,dset=ed_visits2,grpvar=stgroup,outvar=er12_fu,
 grp1name=Control,grp2name=Intervention,title=E.D. Visit Example);
%dominance(figname=AromaTherapy_domdiag,dset=aroma3,grpvar=group,outvar=change,
 grp1name=Alcohol,grp2name=Blend,title=Aroma Therapy Example);
%dominance(figname=Example3_domdiag,dset=example3,grpvar=example3,outvar=value,
 grp1name=Group 1,grp2name=Group 2,title=Counter-example 3);
%dominance(figname=Example4_domdiag,dset=example4,grpvar=example4,outvar=value,
 grp1name=Group 1,grp2name=Group 2,title=Counter-example 4);
%dominance(figname=Example51_domdiag,dset=escher_AB,grpvar=group,outvar=level,
 grp1name=Group A,grp2name=Group B,title=Counter-example 5.1);
%dominance(figname=Example52_domdiag,dset=escher_BC,grpvar=group,outvar=level,
 grp1name=Group B,grp2name=Group C,title=Counter-example 5.2);
%dominance(figname=Example53_domdiag,dset=escher_CD,grpvar=group,outvar=level,
grp1name=Group B,grp2name=Group C,title=Counter-example 5.3);
%dominance(figname=acuity_domdiag,dset=acuity2,grpvar=rp_dom,outvar=acuity,
grp1name=Sex-Linked,grp2name=Dominant,title=Visual Acuity Example);
%dominance(figname=Figure8,dset=toy,grpvar=grp,outvar=y,
grp1name=Y,grp2name=X,title=Figure 8 Toy Example);
title1;
