/* makedata.sas */

/*
 Disclaimer: No guarantee as to suitability or accuracy is
 given or implied. User uses this code entirely at their own risk.
*/

/* Create the E.D. Visits Dataset */
proc format;
  value grped 0='Control' 1='Intervention';
run;
data ed_visits;
  input stGroup  er12_fu Frequency;
  format stGroup grped.;
  label
    er12_fu = 'Number of Emergency Department Visits'
   ;
cards;
0                         0            95
0                         1            13
0                         2             9
0                         3             6
0                         4             1
0                         5             1
0                         7             1
0                         9             2
0                        14             1
1                         0           108
1                         1            14
1                         2             2
1                         3             2
1                         4             2
1                         6             2
1                        20             1
;
run;
/* Get One Unique Observation per Record */
data ed_visits2; set ed_visits;
  do i=1 to frequency;
    output;
   end;
  drop i;
run;

/* Create Aromatherapy Trial Dataset */
proc format;
  value grp 0='Saline' 1='Alcohol' 2='Ginger' 3='Blend';
run;
data aroma;
  input group change n;
  label
    change = 'Change in Post-Op Nausea Score'
   ;
  format group grp.;
cards;
0 -3 0
0 -2 5
0 -1 24
0  0 40
0 +1 4
1 -3 0
1 -2 6 
1 -1 34
1  0 33
1 +1 5
2 -3 2
2 -2 11
2 -1 38
2  0 23
2 +1 2
3 -3 4
3 -2 11
3 -1 46
3  0 12
3 +1 1
;
run;
data aroma2; set aroma;
  if group=1 or group=3;
run;
/* Need One Unique Observation per Record */
data aroma3; set aroma2;
  do i=1 to n;
    output;
   end;
run;

/* Create the dataset(s) for Counterexamples 3 and 4 */
proc format;
 /* The format shift. will be used to shorten the apparent length of the x-axis scale */
  value shift 11='~' 13='99' 14='100' 15='101' 16='102' 17='103' 18='104' 19='105' 20='106' 21='107' ;
 /* The format group3ex. and group4ex will be used to switch which observations are in which group */
  value group3ex 1='A' 2='B' 3='A' 4='B' 5='B' 6='A';
  value group4ex 1='A' 2='B' 3='B' 4='A' 5='A' 6='B';
run;
data example;
  input Sample Value ;
  example3 = sample;
  example4 = sample;
  format
    example3 group3ex.
	example4 group4ex.
	value shift.
   ;
cards;
1 1
1 1
1 2
1 2
1 2
1 3
1 3
2 5 
2 5 
2 6 
2 6
2 6
2 7
2 7
3 9
4 13
5 15
5 15
5 16
5 16
5 16
5 17
5 17
6 19
6 19
6 20
6 20
6 20
6 21
6 21
;
run;
data example3; set example;
  keep example3 value;
  if example3=3 then example3=1;
  if example3=6 then example3=1;
  if example3=4 then example3=2;
  if example3=5 then example3=2;
  format value;
run;
proc print data=example3;
run;
data example4; set example;
  keep example4 value;
  if example4=3 then example4=2;
  if example4=6 then example4=2;
  if example4=4 then example4=1;
  if example4=5 then example4=1;
run;
proc print data=example4;
  format example4;
run;

/* Make the transitivity Example dataset */
proc format;
  value escher 1='A' 2='B' 3='C' 4='D';
run;
data escher;
  input group Level count;
  format group escher.;
cards;
1 1 41
1 7 41
1 8 41
1 9 41
2 2 41
2 3 41
2 10 41
2 11 41
3 4 41
3 5 41
3 6 41
3 12 41
4 1 41
4 7 41
4 8 41
4 9 41
;
run;
data escher2; set escher;
  do i=1 to count;
    output;
   end;
  drop i;
run;
data escher_AB; set escher2;
  if group = 1 or group = 2;
run;
data escher_BC; set escher2;
  if group = 2 or group = 3;
run;
data escher_CD; set escher2;
  if group = 3 or group = 4;
run;

/* Make the "toy" Dataset */
Data toy;
  input grp y ;
datalines;
1 1 
1 1 
1 1 
1 2
2 1 
2 2 
2 3 
2 3 
;
run;

/* Make the Retinitis Pigmentosa (RP) Visual Acuity Dataset for Example From Rosner */
proc format;
  value dom 0='Sex-linked' 1='Dominant' ;
run;
data acuity1 acuity2 dom_acuity sex_acuity;
  input acuity dominant sex_linked;
  output acuity1;
  rp_dom = 1;
  do i=1 to dominant;
    acuity2 = acuity;
	output acuity2;
	output dom_acuity;
   end;
  rp_dom = 0;
  do i=1 to sex_linked;
    acuity2 = acuity;
	output acuity2;
	output sex_acuity;
   end;   
  format rp_dom dom.;
  label
    acuity = "Visual Acuity"
   ;
cards;
20	5	1
25	9	5
30	6	4
40	3	4
50	2	8
60	0	5
70	0	2
80	0	1
;
run;
data acuity2; set acuity2;
  drop dominant sex_linked i;
run;

proc print data=ed_visits;
run;
