/* wmw_estimates_1.sas */
/*  This program computes and outputs statistics related to the Wilcoxon-Mann-Whitney test */
/*  Need to careful about which direction the difference goes! */

/*
 Disclaimer: No guarantee as to suitability or accuracy is
 given or implied. User uses this code entirely at their own risk.
*/

title1 'The SAS System';

%macro doWMW(dset,outvar,grpvar,grp1,grp2,title,verbose);
%if %upcase(&verbose)=NO %then
  %do;
    ods listing close;
  %end;
proc npar1way data=&dset wilcoxon ;
  class &grpvar;
  var &outvar;
  ods output wilcoxonscores=rawwilc kruskalwallistest=kw_pvalue wilcoxontest=wilc_pvalue;
  title5 "&title";
run;
data wil_pvalue; set wilc_pvalue;
  if Name1='P2_WIL';
  keep variable nValue1;
  rename 
    nValue1 = p_value
   ;
run;
title3;

data n1; set rawwilc;
  if _n_=1;
  n1=N;
  class1="&grp1";
  R1=SumofScores;
  MeanRank1=R1/n1;
  link=1;
  if _n_=1 then call symput('g1',class);
  keep variable class1 n1 R1 MeanRank1;
run;
data n2; set rawwilc;
  if _n_=2;
  class2="&grp2";
  n2=N;
  R2=SumofScores;
  MeanRank2=R2/n2;
  link=1;
  if _n_=2 then call symput('g2',class);
  keep variable class2 n2 R2 MeanRank2;
run;
data wilcstuff;
  merge n1 n2 rawwilc wil_pvalue;
  by variable;
  U1 = SumOfScores - N*(N+1)/2;
  U2 = SumOfScores - U1;
  p1 = U1/(n1*n2);
  se_p1 = StdDevOfSum/(n1*n2);
  p1_lower = p1-1.96*se_p1;
  if -999999<p1_lower<0 then p1_lower=0;
  p1_upper = p1+1.96*se_p1;
  if p1_upper > 1 then p1_upper=1;
  if p1^=1 then WMWodds = p1/(1-p1);
  if p1_lower^=1 then WMWodds_lower = p1_lower /(1-p1_lower );
  if p1_upper^=1 then WMWodds_upper = p1_upper /(1-p1_upper );
 /* If you want 1-p1, use _n_=2 in next line? */
  if _n_=1;
  label
    p1="p1 = Prob(&g1 > &g2)"
   ;
run;

proc logistic data=&dset ;
  class &grpvar;
  model &grpvar=&outvar;
  roc;
  ods output rocAssociation=roc_est;
  title3 "Estimates for p'', WMWodds and (SAS ROC based) 95% Confidence Intervals";
  title5 "&title";
run;
data roc_est2; set roc_est;
  if Lowerarea^=1 then WMWodds_Lower_Delong = lowerArea/(1-lowerArea);
  if Upperarea^=1 then WMWodds_Upper_Delong = UpperArea/(1-UpperArea);
  label
    lowerArea = 'p1 Lower Bound (Delong)'
    UpperArea = 'p1 Upper Bound (Delong)'
    WMWodds_Lower_Delong = 'WMWodds Lower Bound (Delong)'
    WMWodds_Upper_Delong = 'WMWodds Upper Bound (Delong)'
   ;
  keep area lowerArea upperArea 
      WMWodds_Lower_Delong WMWodds_Upper_Delong ;
run;
data allest;
  merge wilcstuff roc_est2;
  if 0<=p1<0.5 then do;
    lowerArea2 = 1 - upperArea;
    upperArea = 1 - lowerArea;
	lowerArea = lowerarea2;
	if Upperarea^=1 then WMWodds_Lower_Delong = LowerArea/(1-LowerArea);
    if Upperarea=1 then WMWodds_Lower_Delong = 999; 
    if Lowerarea^=1 then WMWodds_Upper_Delong = UpperArea/(1-UpperArea);
    if Lowerarea=1 then WMWodds_Upper_Delong = 0.1;
   end;
  drop class N SumOfScores ExpectedSum StdDevOfSum MeanScore lowerarea2;
  label
    se_p1 = "StdErr p1"
    p1_lower = "p1 Lower Bound (Test Based)"
    p1_upper = "p1 Upper Bound (Test Based)"
    WMWodds_Lower = 'WMWodds Lower Bound (Test Based)'
    WMWodds_Upper = 'WMWodds Upper Bound (Test Based)'
   ;
run;

options linesize=132;
ods listing;
proc print data=allest noobs ;*label;
  var Variable n1 class1 n2 class2 U1 area p1 se_p1 p1_lower p1_upper lowerArea upperArea
     WMWodds WMWodds_Lower WMWodds_Upper WMWodds_Lower_Delong WMWodds_Upper_Delong p_value;
  format p1 5.3 se_p1 p1_lower p1_upper
     WMWodds WMWodds_Lower WMWodds_Upper WMWodds_Lower_Delong WMWodds_Upper_Delong 5.2 p_value pvalue5.3;
  title3 "Estimates for p'', WMWodds and 95% Confidence Intervals";
  title5 "&title";
  where variable^='';
run;
title3;
%mend doWMW;

%doWMW(dset=ed_visits2,grpvar=stgroup,outvar=er12_fu,
grp1=Control,grp2=Intervention,title=ED Visit Example,verbose=YES);
%doWMW(dset=aroma3,grpvar=group,outvar=change,
grp1=Alcohol,grp2=Blend,title=Aroma Therapy Example,verbose=YES);
%doWMW(dset=example,grpvar=example3,outvar=value,
grp1=,grp2=,title=Counter-example 3,verbose=YES);
%doWMW(dset=example,grpvar=example4,outvar=value,
grp1=,grp2=,title=Counter-example 4,verbose=YES);
%doWMW(dset=escher_AB,grpvar=group,outvar=level,
grp1=,grp2=,title=Counter-example 5.1,verbose=YES);
%doWMW(dset=escher_BC,grpvar=group,outvar=level,
grp1=,grp2=,title=Counter-example 5.2,verbose=YES);
%doWMW(dset=escher_CD,grpvar=group,outvar=level,
grp1=,grp2=,title=Counter-example 5.3,verbose=YES);
%doWMW(dset=acuity2,grpvar=rp_dom,outvar=acuity,
grp1=,grp2=,title=Visual Acuity Example,verbose=YES);
%doWMW(dset=toy,grpvar=grp,outvar=y,
grp1=,grp2=,title=Toy Example,verbose=YES);

/*
proc print data=roc_est;
run;
proc print data=rawwilc;
run;
proc print data=wil_pvalue;
run;
proc print data=allest;
run;
*/
